@@ -1,5 +1,5 @@
 /* cp.c  -- file copying (main routines)
-   Copyright (C) 89, 90, 91, 1995-2002 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2003 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,8 +32,9 @@
 #include "backupfile.h"
 #include "copy.h"
 #include "cp-hash.h"
-#include "error.h"
 #include "dirname.h"
+#include "error.h"
+#include "mmap-stack.h"
 #include "path-concat.h"
 #include "quote.h"
 
@@ -119,9 +120,6 @@ static int const reply_vals[] =
   I_ALWAYS_YES, I_ALWAYS_NO, I_ASK_USER
 };
 
-/* The error code to return to the system. */
-static int exit_status = 0;
-
 static struct option const long_opts[] =
 {
   {"archive", no_argument, NULL, 'a'},
@@ -490,7 +488,7 @@ do_copy (int n_files, char **file, const char *target_directory,
 
   if (n_files <= 0)
     {
-      error (0, 0, _("missing file arguments"));
+      error (0, 0, _("missing file argument"));
       usage (EXIT_FAILURE);
     }
   if (n_files == 1 && !target_directory)
@@ -785,9 +783,9 @@ decode_preserve_arg (char const *arg, struct cp_options *x, int on_off)
       char *comma = strchr (s, ',');
       enum File_attribute val;
 
-      /* put a NUL in its place */
+      /* If we found a comma, put a NUL in its place and advance.  */
       if (comma)
-	*comma = 0;
+	*comma++ = 0;
 
       /* process S.  */
       val = XARGMATCH ("--preserve", s, preserve_args, preserve_vals);
@@ -826,6 +824,25 @@ decode_preserve_arg (char const *arg, struct cp_options *x, int on_off)
   free (arg_writable);
 }
 
+static void run (size_t, char **, char const *, struct cp_options *)
+     ATTRIBUTE_NORETURN;
+/* Encapsulate the `copy-and-exit' functionality.  */
+static void
+run (size_t n_files, char **files, char const *target_directory,
+     struct cp_options *x)
+{
+  int fail;
+
+  /* Allocate space for remembering copied and created files.  */
+  hash_init ();
+
+  fail = do_copy (n_files, files, target_directory, x);
+
+  forget_all ();
+
+  exit (fail ? EXIT_FAILURE : EXIT_SUCCESS);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -1050,13 +1067,6 @@ main (int argc, char **argv)
   if (x.unlink_dest_after_failed_open && (x.hard_link || x.symbolic_link))
     x.unlink_dest_before_opening = 1;
 
-  /* Allocate space for remembering copied and created files.  */
-
-  hash_init ();
-
-  exit_status |= do_copy (argc - optind, argv + optind, target_directory, &x);
-
-  forget_all ();
-
-  exit (exit_status);
+  RUN_WITH_BIG_STACK_4 (run, argc - optind, argv + optind,
+			target_directory, &x);
 }
